"""
Enterprise data api application. This Django app exposes API endpoints used by enterprises.
"""

from __future__ import absolute_import, unicode_literals

__version__ = "1.3.3"

default_app_config = "enterprise_data.apps.EnterpriseDataAppConfig"  # pylint: disable=invalid-name
