# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-31 19:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0012_auto_20180831_1930'),
    ]

    operations = [
        migrations.RenameField(
            model_name='enterpriseenrollment',
            old_name='enterprise_user_id',
            new_name='enterprise_user',
        ),
    ]
